@@ -1103,8 +1103,9 @@ class Daemon(object):
         msg_seq = 0
         try:
             msg = message.Message.recv(conn, [message.MSG_CONNECT], hmac_key=self._pyroHmacKey)
+            msg_seq = msg.seq
             if denied_reason:
-                raise errors.ConnectionClosedError(denied_reason)
+                raise Exception(denied_reason)
             if Pyro4.config.LOGWIRE:
                 _log_wiredata(log, "daemon handshake received", msg)
             if msg.serializer_id not in self.__serializer_ids:
@@ -1114,7 +1115,6 @@ class Daemon(object):
             else:
                 current_context.correlation_id = uuid.uuid1()
             serializer_id = msg.serializer_id
-            msg_seq = msg.seq
             serializer = util.get_serializer_by_id(serializer_id)
             data = serializer.deserializeData(msg.data, msg.flags & Pyro4.message.FLAGS_COMPRESSED)
             handshake_response = self.validateHandshake(conn, data["handshake"])
@@ -1481,6 +1481,17 @@ class Daemon(object):
             loc = self.locationStr
         return URI("PYRO:%s@%s" % (objectOrId, loc))
 
+    def resetMetadataCache(self, objectOrId, nat=True):
+        """Reset cache of metadata when a Daemon has available methods/attributes
+        dynamically updated.  Clients will have to get a new proxy to see changes"""
+        uri = self.uriFor(objectOrId, nat)
+        # can only be cached if registered, else no-op
+        if uri.object in self.objectsById:
+            registered_object = self.objectsById[uri.object]
+            # Clear cache regardless of how it is accessed
+            util.reset_exposed_members(registered_object, Pyro4.config.REQUIRE_EXPOSE, as_lists=True)
+            util.reset_exposed_members(registered_object, Pyro4.config.REQUIRE_EXPOSE, as_lists=False)
+
     def proxyFor(self, objectOrId, nat=True):
         """
         Get a fully initialized Pyro Proxy for the given object (or object id) for this daemon.
